"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ReduxEventStorageAdapter = void 0;

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _core = require("@castore/core");

var _errors = require("./errors");

var _getEventStoreSliceName = require("./utils/getEventStoreSliceName");

var _parseAppliedListAggregateIdsOptions = require("./utils/parseAppliedListAggregateIdsOptions");

var _excluded = ["pageToken"];

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

var hasReduxEventStorageAdapter = function hasReduxEventStorageAdapter(groupedEvent) {
  return groupedEvent.eventStorageAdapter instanceof ReduxEventStorageAdapter;
};

var hasContext = function hasContext(groupedEvent) {
  return groupedEvent.context !== undefined;
};

var parseGroupedEvents = function parseGroupedEvents() {
  var timestampInfos;
  var groupedEvents = [];

  for (var _len = arguments.length, groupedEventsInput = new Array(_len), _key = 0; _key < _len; _key++) {
    groupedEventsInput[_key] = arguments[_key];
  }

  groupedEventsInput.forEach(function (groupedEvent, groupedEventIndex) {
    if (!hasReduxEventStorageAdapter(groupedEvent)) {
      throw new Error("Event group event #".concat(groupedEventIndex, " is not connected to a ReduxEventStorageAdapter"));
    }

    if (!hasContext(groupedEvent)) {
      throw new Error("Event group event #".concat(groupedEventIndex, " misses context"));
    }

    if (groupedEvent.event.timestamp !== undefined && timestampInfos !== undefined) {
      timestampInfos = {
        timestamp: groupedEvent.event.timestamp,
        groupedEventIndex: groupedEventIndex
      };
    }

    groupedEvents.push(groupedEvent);
  });

  if (timestampInfos !== undefined) {
    /**
     * @debt type "strangely, a second const is needed to keep the type as defined in forEach loop"
     */
    var _timestampInfos = timestampInfos;
    groupedEvents.forEach(function (groupedEvent, groupedEventIndex) {
      if (groupedEvent.event.timestamp === undefined) {
        groupedEvent.event.timestamp = _timestampInfos.timestamp;
      } else if (groupedEvent.event.timestamp !== _timestampInfos.timestamp) {
        throw new Error("Event group events #".concat(groupedEventIndex, " and #").concat(_timestampInfos.groupedEventIndex, " have different timestamps"));
      }
    });
  }

  return _objectSpread({
    groupedEvents: groupedEvents
  }, timestampInfos !== undefined ? {
    timestamp: timestampInfos.timestamp
  } : {});
};

var ReduxEventStorageAdapter = /*#__PURE__*/(0, _createClass2["default"])(function ReduxEventStorageAdapter(_ref) {
  var _this = this;

  var store = _ref.store,
      eventStoreId = _ref.eventStoreId,
      prefix = _ref.prefix;
  (0, _classCallCheck2["default"])(this, ReduxEventStorageAdapter);
  (0, _defineProperty2["default"])(this, "getEvents", void 0);
  (0, _defineProperty2["default"])(this, "pushEventSync", void 0);
  (0, _defineProperty2["default"])(this, "pushEvent", void 0);
  (0, _defineProperty2["default"])(this, "pushEventGroup", void 0);
  (0, _defineProperty2["default"])(this, "groupEvent", void 0);
  (0, _defineProperty2["default"])(this, "listAggregateIds", void 0);
  (0, _defineProperty2["default"])(this, "store", void 0);
  (0, _defineProperty2["default"])(this, "eventStoreId", void 0);
  (0, _defineProperty2["default"])(this, "eventStoreSliceName", void 0);
  (0, _defineProperty2["default"])(this, "getEventStoreState", void 0);
  this.store = store;
  this.eventStoreId = eventStoreId;
  this.eventStoreSliceName = (0, _getEventStoreSliceName.getEventStoreSliceName)({
    eventStoreId: eventStoreId,
    prefix: prefix
  });

  this.getEventStoreState = function () {
    var eventStoreSliceName = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _this.eventStoreSliceName;

    var eventStoreState = _this.store.getState()[eventStoreSliceName];

    if (eventStoreState === undefined) throw new _errors.ReduxStateNotFoundError({
      eventStoreSliceName: eventStoreSliceName
    });
    return eventStoreState;
  };

  this.pushEventSync = function (event, options) {
    var _eventStoreState$even;

    var aggregateId = event.aggregateId;
    var _options$force = options.force,
        force = _options$force === void 0 ? false : _options$force;

    var eventStoreState = _this.getEventStoreState();

    var events = (_eventStoreState$even = eventStoreState.eventsByAggregateId[aggregateId]) !== null && _eventStoreState$even !== void 0 ? _eventStoreState$even : [];

    if (!force && events.some(function (_ref2) {
      var version = _ref2.version;
      return version === event.version;
    })) {
      throw new _errors.ReduxStoreEventAlreadyExistsError({
        eventStoreId: _this.eventStoreId,
        aggregateId: event.aggregateId,
        version: event.version
      });
    }

    _this.store.dispatch({
      type: "".concat(_this.eventStoreSliceName, "/eventPushed"),
      payload: event
    });

    return {
      event: event
    };
  };

  this.pushEvent = /*#__PURE__*/function () {
    var _ref3 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(event, context) {
      return _regenerator["default"].wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              return _context.abrupt("return", new Promise(function (resolve) {
                var timestamp = new Date().toISOString();
                resolve(_this.pushEventSync(_objectSpread({
                  timestamp: timestamp
                }, event), context));
              }));

            case 1:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));

    return function (_x, _x2) {
      return _ref3.apply(this, arguments);
    };
  }();

  this.pushEventGroup = /*#__PURE__*/function () {
    var _ref4 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2(options) {
      var _len2,
          groupedEventsInput,
          _key2,
          _args2 = arguments;

      return _regenerator["default"].wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              for (_len2 = _args2.length, groupedEventsInput = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
                groupedEventsInput[_key2 - 1] = _args2[_key2];
              }

              return _context2.abrupt("return", new Promise(function (resolve) {
                var _parseGroupedEvents = parseGroupedEvents.apply(void 0, groupedEventsInput),
                    groupedEvents = _parseGroupedEvents.groupedEvents,
                    _parseGroupedEvents$t = _parseGroupedEvents.timestamp,
                    timestamp = _parseGroupedEvents$t === void 0 ? new Date().toISOString() : _parseGroupedEvents$t;

                var responses = [];

                var _iterator = _createForOfIteratorHelper(groupedEvents),
                    _step;

                try {
                  for (_iterator.s(); !(_step = _iterator.n()).done;) {
                    var groupedEvent = _step.value;
                    var eventStorageAdapter = groupedEvent.eventStorageAdapter,
                        event = groupedEvent.event,
                        context = groupedEvent.context;

                    try {
                      var response = eventStorageAdapter.pushEventSync(_objectSpread({
                        timestamp: timestamp
                      }, event), _objectSpread(_objectSpread({}, options), context));
                      responses.push(response);
                    } catch (error) {
                      (0, _toConsumableArray2["default"])(groupedEvents).slice(0, responses.length) // Revert it in reversed order
                      .reverse().forEach(function (groupedEventToRevert) {
                        var eventToRevertEventStorageAdapter = groupedEventToRevert.eventStorageAdapter,
                            eventToRevert = groupedEventToRevert.event;
                        var aggregateId = eventToRevert.aggregateId,
                            version = eventToRevert.version;
                        eventToRevertEventStorageAdapter.store.dispatch({
                          type: "".concat(eventToRevertEventStorageAdapter.eventStoreSliceName, "/eventReverted"),
                          payload: {
                            aggregateId: aggregateId,
                            version: version
                          }
                        });
                      });
                      throw error;
                    }
                  }
                } catch (err) {
                  _iterator.e(err);
                } finally {
                  _iterator.f();
                }

                resolve({
                  eventGroup: responses
                });
              }));

            case 2:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2);
    }));

    return function (_x3) {
      return _ref4.apply(this, arguments);
    };
  }();

  this.groupEvent = function (event) {
    return new _core.GroupedEvent({
      event: event,
      eventStorageAdapter: _this
    });
  };

  this.getEvents = function (aggregateId, _) {
    var _ref5 = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
        minVersion = _ref5.minVersion,
        maxVersion = _ref5.maxVersion,
        reverse = _ref5.reverse,
        limit = _ref5.limit;

    return new Promise(function (resolve) {
      var _eventStoreState$even2;

      var eventStoreState = _this.getEventStoreState();

      var events = (_eventStoreState$even2 = eventStoreState.eventsByAggregateId[aggregateId]) !== null && _eventStoreState$even2 !== void 0 ? _eventStoreState$even2 : [];

      if (minVersion !== undefined) {
        events = events.filter(function (_ref6) {
          var version = _ref6.version;
          return version >= minVersion;
        });
      }

      if (maxVersion !== undefined) {
        events = events.filter(function (_ref7) {
          var version = _ref7.version;
          return version <= maxVersion;
        });
      }

      if (reverse === true) {
        events = events.reverse();
      }

      if (limit !== undefined) {
        events = events.slice(0, limit);
      }

      resolve({
        events: events
      });
    });
  };

  this.listAggregateIds = function (_) {
    var _ref8 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
        inputPageToken = _ref8.pageToken,
        inputOptions = (0, _objectWithoutProperties2["default"])(_ref8, _excluded);

    return new Promise(function (resolve) {
      var eventStoreState = _this.getEventStoreState();

      var _parseAppliedListAggr = (0, _parseAppliedListAggregateIdsOptions.parseAppliedListAggregateIdsOptions)({
        inputPageToken: inputPageToken,
        inputOptions: inputOptions
      }),
          limit = _parseAppliedListAggr.limit,
          initialEventAfter = _parseAppliedListAggr.initialEventAfter,
          initialEventBefore = _parseAppliedListAggr.initialEventBefore,
          reverse = _parseAppliedListAggr.reverse,
          exclusiveStartKey = _parseAppliedListAggr.exclusiveStartKey;

      var aggregateIds = (0, _toConsumableArray2["default"])(eventStoreState.aggregateIds).sort(function (aggregateA, aggregateB) {
        return aggregateA.initialEventTimestamp > aggregateB.initialEventTimestamp ? 1 : -1;
      });

      if (initialEventAfter !== undefined) {
        aggregateIds = aggregateIds.filter(function (_ref9) {
          var initialEventTimestamp = _ref9.initialEventTimestamp;
          return initialEventAfter <= initialEventTimestamp;
        });
      }

      if (initialEventBefore !== undefined) {
        aggregateIds = aggregateIds.filter(function (_ref10) {
          var initialEventTimestamp = _ref10.initialEventTimestamp;
          return initialEventTimestamp <= initialEventBefore;
        });
      }

      if (reverse === true) {
        aggregateIds = aggregateIds.reverse();
      }

      if (exclusiveStartKey !== undefined) {
        var exclusiveStartKeyIndex = aggregateIds.findIndex(function (_ref11) {
          var aggregateId = _ref11.aggregateId;
          return aggregateId === exclusiveStartKey.aggregateId;
        });
        aggregateIds = aggregateIds.slice(exclusiveStartKeyIndex + 1);
      }

      var numberOfAggregateIdsBeforeLimit = aggregateIds.length;

      if (limit !== undefined) {
        aggregateIds = aggregateIds.slice(0, limit);
      }

      var hasNextPage = limit === undefined ? false : numberOfAggregateIdsBeforeLimit > limit;
      var parsedNextPageToken = {
        limit: limit,
        initialEventAfter: initialEventAfter,
        initialEventBefore: initialEventBefore,
        reverse: reverse,
        lastEvaluatedKey: aggregateIds[aggregateIds.length - 1]
      };
      resolve(_objectSpread({
        aggregateIds: aggregateIds
      }, hasNextPage ? {
        nextPageToken: JSON.stringify(parsedNextPageToken)
      } : {}));
    });
  };
});
exports.ReduxEventStorageAdapter = ReduxEventStorageAdapter;
//# sourceMappingURL=adapter.js.map